@@ -22,9 +22,13 @@
 
 #include "agenttype_p.h"
 #include "agentinstance_p.h"
+#include "dbusconnectionpool.h"
+
+#include <akonadi/private/protocol_p.h>
 
 #include "collection.h"
 
+#include <QtDBus/QDBusServiceWatcher>
 #include <QtGui/QWidget>
 
 #include <KGlobal>
@@ -92,12 +96,11 @@ void AgentManagerPrivate::agentInstanceAdded( const QString &identifier )
     // The server-side agent manager then emits the instance added signal when
     // the D-Bus interface for the agent comes up.
     // In this case, we simply notify that the instance status has changed.
-    bool newAgentInstance = !mInstances.contains( identifier );
+    const bool newAgentInstance = !mInstances.contains( identifier );
     if ( newAgentInstance ) {
       mInstances.insert( identifier, instance );
       emit mParent->instanceAdded( instance );
-    }
-    else {
+    } else {
       mInstances.remove( identifier );
       mInstances.insert( identifier, instance );
       emit mParent->instanceStatusChanged( instance );
@@ -180,7 +183,7 @@ void AgentManagerPrivate::agentInstanceNameChanged( const QString &identifier, c
 
 void AgentManagerPrivate::readAgentTypes()
 {
-  QDBusReply<QStringList> types = mManager->agentTypes();
+  const QDBusReply<QStringList> types = mManager->agentTypes();
   if ( types.isValid() ) {
     foreach ( const QString &type, types.value() ) {
       if ( !mTypes.contains( type ) )
@@ -189,6 +192,18 @@ void AgentManagerPrivate::readAgentTypes()
   }
 }
 
+void AgentManagerPrivate::readAgentInstances()
+{
+  const QDBusReply<QStringList> instances = mManager->agentInstances();
+  if ( instances.isValid() ) {
+    foreach ( const QString &instance, instances.value() ) {
+      if ( !mInstances.contains( instance ) ) {
+        agentInstanceAdded( instance );
+      }
+    }
+  }
+}
+
 AgentType AgentManagerPrivate::fillAgentType( const QString &identifier ) const
 {
   AgentType type;
@@ -263,53 +278,81 @@ AgentInstance AgentManagerPrivate::fillAgentInstanceLight( const QString &identi
   return instance;
 }
 
+void AgentManagerPrivate::serviceOwnerChanged( const QString&, const QString &oldOwner, const QString& )
+{
+  if ( oldOwner.isEmpty() ) {
+    readAgentTypes();
+    readAgentInstances();
+  }
+}
+
+void AgentManagerPrivate::createDBusInterface()
+{
+  mTypes.clear();
+  mInstances.clear();
+  delete mManager;
+
+  mManager = new org::freedesktop::Akonadi::AgentManager( QLatin1String( AKONADI_DBUS_CONTROL_SERVICE ),
+                                                          QLatin1String( "/AgentManager" ),
+                                                          DBusConnectionPool::threadConnection(), mParent );
+
+  QObject::connect( mManager, SIGNAL(agentTypeAdded(QString)),
+                    mParent, SLOT(agentTypeAdded(QString)) );
+  QObject::connect( mManager, SIGNAL(agentTypeRemoved(QString)),
+                    mParent, SLOT(agentTypeRemoved(QString)) );
+  QObject::connect( mManager, SIGNAL(agentInstanceAdded(QString)),
+                    mParent, SLOT(agentInstanceAdded(QString)) );
+  QObject::connect( mManager, SIGNAL(agentInstanceRemoved(QString)),
+                    mParent, SLOT(agentInstanceRemoved(QString)) );
+  QObject::connect( mManager, SIGNAL(agentInstanceStatusChanged(QString,int,QString)),
+                    mParent, SLOT(agentInstanceStatusChanged(QString,int,QString)) );
+  QObject::connect( mManager, SIGNAL(agentInstanceProgressChanged(QString,uint,QString)),
+                    mParent, SLOT(agentInstanceProgressChanged(QString,uint,QString)) );
+  QObject::connect( mManager, SIGNAL(agentInstanceNameChanged(QString,QString)),
+                    mParent, SLOT(agentInstanceNameChanged(QString,QString)) );
+  QObject::connect( mManager, SIGNAL(agentInstanceWarning(QString,QString)),
+                    mParent, SLOT(agentInstanceWarning(QString,QString)) );
+  QObject::connect( mManager, SIGNAL(agentInstanceError(QString,QString)),
+                    mParent, SLOT(agentInstanceError(QString,QString)) );
+  QObject::connect( mManager, SIGNAL(agentInstanceOnlineChanged(QString,bool)),
+                    mParent, SLOT(agentInstanceOnlineChanged(QString,bool)) );
+
+  if ( mManager->isValid() ) {
+    QDBusReply<QStringList> result = mManager->agentTypes();
+    if ( result.isValid() ) {
+      foreach ( const QString &type, result.value() ) {
+        const AgentType agentType = fillAgentType( type );
+        mTypes.insert( type, agentType );
+      }
+    }
+    result = mManager->agentInstances();
+    if ( result.isValid() ) {
+      foreach ( const QString &instance, result.value() ) {
+        const AgentInstance agentInstance = fillAgentInstance( instance );
+        mInstances.insert( instance, agentInstance );
+      }
+    }
+  } else {
+    kWarning() << "AgentManager failed to get a valid AgentManager DBus interface. Error is:" << mManager->lastError().type() << mManager->lastError().name() << mManager->lastError().message();
+  }
+}
+
 AgentManager* AgentManagerPrivate::mSelf = 0;
 
 AgentManager::AgentManager()
   : QObject( 0 ), d( new AgentManagerPrivate( this ) )
 {
-  d->mManager = new org::freedesktop::Akonadi::AgentManager( QLatin1String( "org.freedesktop.Akonadi.Control" ),
-                                                     QLatin1String( "/AgentManager" ),
-                                                     QDBusConnection::sessionBus(), this );
+  // needed for queued connections on our signals
+  qRegisterMetaType<Akonadi::AgentType>();
+  qRegisterMetaType<Akonadi::AgentInstance>();
 
-  connect( d->mManager, SIGNAL( agentTypeAdded( const QString& ) ),
-           this, SLOT( agentTypeAdded( const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentTypeRemoved( const QString& ) ),
-           this, SLOT( agentTypeRemoved( const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentInstanceAdded( const QString& ) ),
-           this, SLOT( agentInstanceAdded( const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentInstanceRemoved( const QString& ) ),
-           this, SLOT( agentInstanceRemoved( const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentInstanceStatusChanged( const QString&, int, const QString& ) ),
-           this, SLOT( agentInstanceStatusChanged( const QString&, int, const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentInstanceProgressChanged( const QString&, uint, const QString& ) ),
-           this, SLOT( agentInstanceProgressChanged( const QString&, uint, const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentInstanceNameChanged( const QString&, const QString& ) ),
-           this, SLOT( agentInstanceNameChanged( const QString&, const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentInstanceWarning( const QString&, const QString& ) ),
-           this, SLOT( agentInstanceWarning( const QString&, const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentInstanceError( const QString&, const QString& ) ),
-           this, SLOT( agentInstanceError( const QString&, const QString& ) ) );
-  connect( d->mManager, SIGNAL( agentInstanceOnlineChanged( const QString&, bool ) ),
-           this, SLOT( agentInstanceOnlineChanged( const QString&, bool ) ) );
+  d->createDBusInterface();
 
-  if ( d->mManager->isValid() ) {
-    QDBusReply<QStringList> result = d->mManager->agentTypes();
-    if ( result.isValid() ) {
-      foreach( const QString &type, result.value() ) {
-        const AgentType agentType = d->fillAgentType( type );
-        d->mTypes.insert( type, agentType );
-      }
-    }
-
-    result = d->mManager->agentInstances();
-    if ( result.isValid() ) {
-      foreach( const QString &instance, result.value() ) {
-        const AgentInstance agentInstance = d->fillAgentInstance( instance );
-        d->mInstances.insert( instance, agentInstance );
-      }
-    }
-  }
+  QDBusServiceWatcher *watcher = new QDBusServiceWatcher( QLatin1String( AKONADI_DBUS_CONTROL_SERVICE ),
+                                                          DBusConnectionPool::threadConnection(),
+                                                          QDBusServiceWatcher::WatchForOwnerChange, this );
+  connect( watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+           this, SLOT(serviceOwnerChanged(QString,QString,QString)) );
 }
 
 // @endcond
@@ -352,11 +395,16 @@ void AgentManager::removeInstance( const AgentInstance &instance )
   d->mManager->removeAgentInstance( instance.identifier() );
 }
 
-void AgentManager::synchronizeCollection(const Collection & collection)
+void AgentManager::synchronizeCollection( const Collection & collection )
+{
+  synchronizeCollection( collection, false );
+}
+
+void AgentManager::synchronizeCollection( const Collection & collection, bool recursive )
 {
   const QString resId = collection.resource();
   Q_ASSERT( !resId.isEmpty() );
-  d->mManager->agentInstanceSynchronizeCollection( resId, collection.id() );
+  d->mManager->agentInstanceSynchronizeCollection( resId, collection.id(), recursive );
 }
 
 #include "agentmanager.moc"
