@@ -82,7 +82,6 @@ extern "C" {
 #include <kdebug.h>
 #include <kio/connection.h>
 #include <kio/slaveinterface.h>
-#include <kio/passworddialog.h>
 #include <klocale.h>
 #include <kmimetype.h>
 #include <kcodecs.h>
@@ -1231,7 +1230,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     KUrl src;
     KUrl dest;
     stream >> src >> dest;
-    copy(src, dest, 0, false);
+    copy(src, dest, 0, KIO::DefaultFlags);
     break;
   }
   case 'c':
@@ -2577,7 +2576,7 @@ ssize_t IMAP4Protocol::myRead(void *data, ssize_t len)
     ssize_t copyLen = (len < readBufferLen) ? len : readBufferLen;
     memcpy(data, readBuffer, copyLen);
     readBufferLen -= copyLen;
-    if (readBufferLen) memcpy(readBuffer, &readBuffer[copyLen], readBufferLen);
+    if (readBufferLen) memmove(readBuffer, &readBuffer[copyLen], readBufferLen);
     return copyLen;
   }
   if (!isConnected()) return 0;
