@@ -22,6 +22,7 @@
 */
 #include "kmime_parsers.h"
 
+#include <QtCore/QRegExp>
 #include <QtCore/QByteArray>
 
 using namespace KMime::Parser;
@@ -58,7 +59,7 @@ bool MultiPart::parse()
       // this message is *really* broken
       pos1 = -1; //we give up
     } else if ( ( pos1 - blen ) > 1 ) { //preamble present
-      p_reamble = s_rc.left( pos1 - blen );
+      p_reamble = s_rc.left( pos1 - blen - 1 );
     }
   }
 
@@ -163,7 +164,7 @@ QByteArray NonMimeParser::guessMimeType( const QByteArray &fileName )
 //==============================================================================
 
 UUEncoded::UUEncoded( const QByteArray &src, const QByteArray &subject ) :
-  NonMimeParser( src ), s_ubject( subject ), m_beginRegExp( "begin [0-9][0-9][0-9]" ), m_numberRegExp( "[0-9]+/[0-9]+" )
+  NonMimeParser( src ), s_ubject( subject )
 {}
 
 bool UUEncoded::parse()
@@ -176,8 +177,8 @@ bool UUEncoded::parse()
     bool containsBegin=false, containsEnd=false;
     QByteArray tmp, fileName;
 
-    if ( ( beginPos = QString( s_rc ).
-           indexOf( m_beginRegExp, currentPos ) ) > -1 &&
+    if ( ( beginPos = QString::fromLatin1( s_rc ).indexOf( QRegExp( QLatin1String( "begin [0-9][0-9][0-9]" ) ),
+                                                           currentPos ) ) > -1 &&
          ( beginPos == 0 || s_rc.at( beginPos - 1 ) == '\n') ) {
       containsBegin = true;
       uuStart = s_rc.indexOf( '\n', beginPos );
@@ -215,15 +216,16 @@ bool UUEncoded::parse()
       //printf("lineCount=%d , MCount=%d\n", lineCount, MCount);
       if ( MCount == 0 || ( lineCount - MCount ) > 10 ||
            ( ( !containsBegin || !containsEnd ) && ( MCount < 15 ) ) ) {
-        // harder check for splitted-articles
+        // harder check for split-articles
         success = false;
         break; //too many "non-M-Lines" found, we give up
       }
 
       if ( ( !containsBegin || !containsEnd ) && !s_ubject.isNull() ) {
         // message may be split up => parse subject
-        pos = m_numberRegExp.indexIn( QString( s_ubject ), 0 );
-        len = m_numberRegExp.matchedLength();
+        QRegExp rx( QLatin1String( "[0-9]+/[0-9]+") );
+        pos = rx.indexIn( QLatin1String( s_ubject ), 0 );
+        len = rx.matchedLength();
         if ( pos != -1 ) {
           tmp = s_ubject.mid( pos, len );
           pos = tmp.indexOf( '/' );
